/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.ow2.petals.registry_overlay.configuration.generated.ObjectFactory;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * The topology builder allows to unmarshall the resource associated to the
 * definition of a registry overlay topology.
 * 
 * @author Christophe DENEUX - Linagora
 */
public final class PetalsRegistryOverlayCfgBuilder {

    /**
     * XSD schema
     */
    public static final String NAMESPACE_XSD = "http://www.w3.org/2001/XMLSchema";

    /**
     * Topology schema
     */
    public static final String PETALS_REGISTRY_OVERLAY_XSD = "petalsRegistryOverlayCfgModel.xsd";

    /**
     * The exception raised during JAXB context creation
     */
    private static PetalsRegistryOverlayCfgException jaxbContextCreationEx;

    /**
     * The unmarshaller.
     */
    private static Unmarshaller unmarshaller;

    /**
     * The exception raised during JAXB unique unmarshaller creation.
     */
    private static PetalsRegistryOverlayCfgException unmarshCreationEx;

    /**
     * The marshaller.
     */
    private static Marshaller marshaller;
    
    /**
     * The exception raised during JAXB unique marshaller creation.
     */
    private static PetalsRegistryOverlayCfgException marshCreationEx;

    /**
     * Static object initialization
     */
    static {

        final InputStream schemaInputStream = PetalsRegistryOverlayCfgBuilder.class.getResourceAsStream("/"
                + PetalsRegistryOverlayCfgBuilder.PETALS_REGISTRY_OVERLAY_XSD);

        try {
            final SchemaFactory factory = SchemaFactory.newInstance(PetalsRegistryOverlayCfgBuilder.NAMESPACE_XSD);
            final Source[] schemas = new StreamSource[] { new StreamSource(schemaInputStream) };
            final Schema schema = factory.newSchema(schemas);

            try {
                final JAXBContext jaxbContext = JAXBContext
                        .newInstance(new Class[] { PetalsRegistryOverlay.class });

                try {

                    PetalsRegistryOverlayCfgBuilder.unmarshaller = jaxbContext.createUnmarshaller();
                    PetalsRegistryOverlayCfgBuilder.unmarshaller.setSchema(schema);

                } catch (final JAXBException ex) {
                    PetalsRegistryOverlayCfgBuilder.unmarshCreationEx = new PetalsRegistryOverlayCfgException(
                            "Failed to create the JAXB unmarshaller", ex);
                }

                try {

                    PetalsRegistryOverlayCfgBuilder.marshaller = jaxbContext.createMarshaller();
                    PetalsRegistryOverlayCfgBuilder.marshaller.setSchema(schema);
                    PetalsRegistryOverlayCfgBuilder.marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
                    PetalsRegistryOverlayCfgBuilder.marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT,
                            Boolean.TRUE);

                } catch (final JAXBException ex) {
                    PetalsRegistryOverlayCfgBuilder.marshCreationEx = new PetalsRegistryOverlayCfgException(
                            "Failed to create the JAXB marshaller", ex);
                }
            } catch (JAXBException ex) {
                PetalsRegistryOverlayCfgBuilder.jaxbContextCreationEx = new PetalsRegistryOverlayCfgException(
                        "Failed to create the JAXB context", ex);
            }

        } catch (final SAXException e) {
            final PetalsRegistryOverlayCfgException schemaCreationEx = new PetalsRegistryOverlayCfgException(
                    "Failed to parse resource '" + PetalsRegistryOverlayCfgBuilder.PETALS_REGISTRY_OVERLAY_XSD
                            + "'", e);
            PetalsRegistryOverlayCfgBuilder.jaxbContextCreationEx = new PetalsRegistryOverlayCfgException(
                    "Failed to create the JAXB context", schemaCreationEx);
            PetalsRegistryOverlayCfgBuilder.unmarshCreationEx = new PetalsRegistryOverlayCfgException(
                    "Failed to create the JAXB unmarshaller", schemaCreationEx);
            PetalsRegistryOverlayCfgBuilder.marshCreationEx = new PetalsRegistryOverlayCfgException(
                    "Failed to create the JAXB marshaller", schemaCreationEx);
        }
    }

    /**
     * No constructor available.
     */
    private PetalsRegistryOverlayCfgBuilder() {
        // NOP
    }

    /**
     * Create the Petals Registry Overlay configuration object.
     * 
     * @param xmlFileName
     *            The XML file name containing the Petals Registry Overlay
     *            configuration.
     * @return The Petals Registry Overlay configuration.
     * @throws PetalsRegistryOverlayCfgException
     *             Impossible to create the Petals Registry Overlay
     *             configuration.
     */
    public static PetalsRegistryOverlay createPetalsRegistryOverlayCfg(final String xmlFileName)
            throws PetalsRegistryOverlayCfgException {
        return PetalsRegistryOverlayCfgBuilder.createPetalsRegistryOverlayCfg(new File(xmlFileName));
    }

    /**
     * Create the Petals Registry Overlay configuration object.
     * 
     * @param xmlFile
     *            The XML file containing the Petals Registry Overlay
     *            configuration.
     * @return The Petals Registry Overlay configuration.
     * @throws PetalsRegistryOverlayCfgException
     *             Impossible to create the Petals Registry Overlay
     *             configuration.
     */
    public static PetalsRegistryOverlay createPetalsRegistryOverlayCfg(final File xmlFile)
            throws PetalsRegistryOverlayCfgException {
        try {
            final FileInputStream fis = new FileInputStream(xmlFile);
            try {
                return PetalsRegistryOverlayCfgBuilder.createPetalsRegistryOverlayCfg(fis);
            } finally {
                try {
                    fis.close();
                } catch (final IOException e) {
                    // NOP
                }
            }
        } catch (final FileNotFoundException e) {
            throw new PetalsRegistryOverlayCfgException("The file '" + xmlFile.getAbsolutePath()
                    + "' does not exist", e);
        }
    }

    /**
     * Create the Petals Registry Overlay configuration object.
     * 
     * @param inputStrem
     *            An input stream containing the XML instance of the Petals
     *            Registry Overlay configuration. The input stream <b>MUST</b>
     *            be closed by the caller.
     * @return The Petals Registry Overlay configuration.
     * @throws PetalsRegistryOverlayCfgException
     *             Impossible to create the Petals Registry Overlay
     *             configuration.
     */
    public static PetalsRegistryOverlay createPetalsRegistryOverlayCfg(final InputStream inputStream)
            throws PetalsRegistryOverlayCfgException {

        return PetalsRegistryOverlayCfgBuilder.createPetalsRegistryOverlayCfg(new StreamSource(inputStream));
    }

    /**
     * Create the Petals Registry Overlay configuration object.
     * 
     * @param source
     *            An {@link Source} containing the XML instance of the Petals Registry Overlay configuration.
     * @return The Petals Registry Overlay configuration.
     * @throws PetalsRegistryOverlayCfgException
     *             Impossible to create the Petals Registry Overlay configuration.
     */
    public static PetalsRegistryOverlay createPetalsRegistryOverlayCfg(final Source source)
            throws PetalsRegistryOverlayCfgException {

        if (PetalsRegistryOverlayCfgBuilder.unmarshaller == null) {
            throw PetalsRegistryOverlayCfgBuilder.unmarshCreationEx;
        }
        
        try {
            final JAXBElement<PetalsRegistryOverlay> root;
            // The default Xerces unmarshaller is not thread safe
            synchronized (PetalsRegistryOverlayCfgBuilder.unmarshaller) {
                root = PetalsRegistryOverlayCfgBuilder.unmarshaller.unmarshal(source,
                        PetalsRegistryOverlay.class);
            }

            return root.getValue();
        } catch (final JAXBException e) {
            throw new PetalsRegistryOverlayCfgException(
                    "Failed to build the Petals Registry Overlay configuration", e);
        }
    }

    /**
     * Write the configuration of the Petals Registry Overlay into an xml {@link File}.
     * 
     * @param petalsRegistryOverlayCfg
     *            The configuration of the Petals Registry Overlay
     * @param xmlFile
     *            The {@link File} in which the configuration will be marshaled
     */
    public synchronized static void marshallPetalsRegistryOverlayConf(
            final PetalsRegistryOverlay petalsRegistryOverlayCfg, final File xmlFile)
            throws PetalsRegistryOverlayCfgException {

        if (PetalsRegistryOverlayCfgBuilder.marshaller == null) {
            throw PetalsRegistryOverlayCfgBuilder.marshCreationEx;
        }
        
        try {
            final ObjectFactory jaxbObjectFactory = new ObjectFactory();

            final JAXBElement<PetalsRegistryOverlay> topologyElement = jaxbObjectFactory
                    .createPetalsRegistryOverlay(petalsRegistryOverlayCfg);

            PetalsRegistryOverlayCfgBuilder.marshaller.marshal(topologyElement, xmlFile);
        } catch (JAXBException ex) {
            throw new PetalsRegistryOverlayCfgException(
                    "Failed to write the configuration of the Petals Registry Overlay on disk", ex);
        }
    }

    /**
     * Write the configuration of the Petals Registry Overlay into an DOM {@link Document}
     * 
     * @param petalsRegistryOverlayCfg
     *            The configuration of the Petals Registry Overlay
     * @param document
     *            The target DOM {@link Document} in which the configuration will be marshaled
     */
    public synchronized static void marshallPetalsRegistryOverlayConf(
            final PetalsRegistryOverlay petalsRegistryOverlayCfg, final Document document)
            throws PetalsRegistryOverlayCfgException {

        if (PetalsRegistryOverlayCfgBuilder.marshaller == null) {
            throw PetalsRegistryOverlayCfgBuilder.marshCreationEx;
        }
        
        try {
            final ObjectFactory jaxbObjectFactory = new ObjectFactory();

            final JAXBElement<PetalsRegistryOverlay> petalsRegistryOverlayCfgElement = jaxbObjectFactory
                    .createPetalsRegistryOverlay(petalsRegistryOverlayCfg);

            final DOMResult domResult = new DOMResult(document);
            PetalsRegistryOverlayCfgBuilder.marshaller.marshal(petalsRegistryOverlayCfgElement, domResult);

        } catch (JAXBException ex) {
            throw new PetalsRegistryOverlayCfgException(
                    "Failed to write the configuration of the Petals Registry Overlay on disk", ex);
        }
    }
}
