/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.topology;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.URISyntaxException;
import java.util.List;

import javax.xml.transform.dom.DOMSource;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.registry_overlay.configuration.PetalsRegistryOverlayCfgBuilder;
import org.ow2.petals.registry_overlay.configuration.PetalsRegistryOverlayCfgException;
import org.ow2.petals.registry_overlay.configuration.generated.Credentials;
import org.ow2.petals.registry_overlay.configuration.generated.ManagementConsole;
import org.ow2.petals.registry_overlay.configuration.generated.Member;
import org.ow2.petals.registry_overlay.configuration.generated.Members;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * Unit tests of the {@link PetalsRegistryOverlayCfgBuilder}.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class PetalsRegistryOverlayCfgBuilderTest {

    @Rule
    final public TemporaryFolder temporaryFolder = new TemporaryFolder();

    /**
     * Unit test of the {@link PetalsRegistryOverlayCfgBuilder} about the parsing of a file.
     */
    @Test
    public void testTopologyBuilder_Unmarshalling_00() throws PetalsRegistryOverlayCfgException,
            URISyntaxException {
        final String fileName = new File(Thread.currentThread().getContextClassLoader()
                .getResource("petalsRegistryOverlay.xml").toURI()).getAbsolutePath();

        final PetalsRegistryOverlay registryCfg = PetalsRegistryOverlayCfgBuilder
                .createPetalsRegistryOverlayCfg(fileName);

        assertNotNull("Petals registry overlay is null", registryCfg);
        final Credentials credentials = registryCfg.getCredentials();
        assertNotNull("No credentials", credentials);
        assertEquals("Unexpected value for group", "my-group", credentials.getGroup());
        assertEquals("Unexpected value for password", "my-password", credentials.getPassword());
        assertNotNull("No members", registryCfg.getMembers());
        final List<Member> members = registryCfg.getMembers().getMember();
        assertNotNull("No list member", members);
        assertEquals("Unexpected number of members", 5, members.size());
        final ManagementConsole mngtConsole = registryCfg.getManagementConsole();
        assertNotNull("No management console", mngtConsole);
        assertTrue("Management console disabled", mngtConsole.isEnable());
        assertNotNull("No management URL", mngtConsole.getValue());
    }

    /**
     * Unit test of the {@link PetalsRegistryOverlayCfgBuilder} about the parsing of an input
     * stream
     */
    @Test
    public void testTopologyBuilder_Unmarshalling_01() throws PetalsRegistryOverlayCfgException {
        final InputStream isTopo = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("petalsRegistryOverlay.xml");
        assertNotNull(isTopo);
        PetalsRegistryOverlayCfgBuilder.createPetalsRegistryOverlayCfg(isTopo);
    }

    /**
     * Unit test of the {@link PetalsRegistryOverlayCfgBuilder} about the parsing of a file
     * containing errors
     */
    @Test(expected = PetalsRegistryOverlayCfgException.class)
    public void testTopologyBuilder_Unmarshalling_Fail() throws URISyntaxException,
            PetalsRegistryOverlayCfgException {
        final String fileName = new File(Thread.currentThread().getContextClassLoader()
                .getResource("petalsRegistryOverlayError.xml").toURI()).getAbsolutePath();
        PetalsRegistryOverlayCfgBuilder.createPetalsRegistryOverlayCfg(fileName);
    }

    /**
     * Unit test of the {@link PetalsRegistryOverlayCfgBuilder} about writing a
     * configuration of the Petals Registry Overlay to a file
     */
    @Test
    public void testTopologyBuilder_Marshalling_File_00() throws PetalsRegistryOverlayCfgException,
            IOException {

        final PetalsRegistryOverlay proCfg = new PetalsRegistryOverlay();

        final Credentials credentials = new Credentials();
        credentials.setPassword("my-password");
        credentials.setGroup("my-esb-topo");
        proCfg.setCredentials(credentials);

        final Member member = new Member();
        member.setId("my-member-id");
        member.setPort(new BigInteger("12345"));
        member.setValue("my-member");
        final Members members = new Members();
        members.getMember().add(member);
        proCfg.setMembers(members);

        final File outputFile = this.temporaryFolder.newFile("petalsRegistryOverlay.xml");
        PetalsRegistryOverlayCfgBuilder.marshallPetalsRegistryOverlayConf(proCfg, outputFile);

        final PetalsRegistryOverlay rereadCfg = PetalsRegistryOverlayCfgBuilder
                .createPetalsRegistryOverlayCfg(outputFile);
        assertEquals("Unexpected Petals Registry Overlay configuration was parsed", proCfg, rereadCfg);
    }

    /**
     * Unit test of the {@link PetalsRegistryOverlayCfgBuilder} about writing a configuration of the Petals Registry
     * Overlay as DOM Document
     */
    @Test
    public void testTopologyBuilder_Marshalling_DOM_00() throws PetalsRegistryOverlayCfgException, IOException {

        final PetalsRegistryOverlay proCfg = new PetalsRegistryOverlay();

        final Credentials credentials = new Credentials();
        credentials.setPassword("my-password");
        credentials.setGroup("my-esb-topo");
        proCfg.setCredentials(credentials);

        final Member member = new Member();
        member.setId("my-member-id");
        member.setPort(new BigInteger("12345"));
        member.setValue("my-member");
        final Members members = new Members();
        members.getMember().add(member);
        proCfg.setMembers(members);

        final Document doc = DocumentBuilders.newDocument();
        PetalsRegistryOverlayCfgBuilder.marshallPetalsRegistryOverlayConf(proCfg, doc);

        final PetalsRegistryOverlay rereadCfg = PetalsRegistryOverlayCfgBuilder
                .createPetalsRegistryOverlayCfg(new DOMSource(doc));
        assertEquals("Unexpected Petals Registry Overlay configuration was parsed", proCfg,
                rereadCfg);
    }
}
